% GSEIR_AltPol_US: impact of a alternative policies seeded to US in 3/24/20
% This is a generalized SEIR model with hospitalization, fatalities, and
% quarantine/contact tracing/testing for time-varying parameters.
% Essentially this is a simplified version of the Imperial College
% pandemic model. Unless otherwise noted the model is calibrated based on
% the IC study. 
%
% Option: To run it with standard R0=2.4 from IC2020 or high R0=5.7 from
% Sanche et al (2020)
%
% Variables are
% 1 S:   susceptible population share
% 2 E:   exposed, not infectious, does not test positive
% 3 I1:  infected population share, asymptomatic
% 4 Ee:  known exposed
% 5 I1e: known asymptomatic infected
% 6 I2:  symptomatic infected, not hospitalized, but can be quarantined
% 7 H1:  symptomatic and hospitalized, not requiring ICU
% 8 H2:  symptomatic and hospitalized, requiring ICU
% 9 R:   recovered population share
% 10 D:  fatality share, cumulative
% 11 FE: total tests, cumulative 
%
% (0) M  = a*S*Ieff  and 
%     Ieff= I1 + (1-eTQ)*(Ee+I1e) + s*[(1-eI2)*I2 + (1-eH1)*H1 + (1-eH2)*H2] 
% (1) S' =-M
% (2) E' =+M - phi*E                        - fTE -fTFE
% (3) I1'=   + phi*E             -(b+cI1)*I1 - fTI1-fTFI1-fF 
% (4) Ee' =fTE +fTFE      -phi*Ee
% (5) I1e'=fTI1+fTFI1+ fF +phi*Ee-(b+cI1)*I1e
% (6) I2'=    (1-o)*b*(I1+I1e) -  (cI2+d)*I2 
% (7) H1'= (1-p2)*o*b*(I1+I1e) -  (cH1+d)*H1
% (8) H2'=     p2*o*b*(I1+I1e) -  (cH2+dH2)*H2
% (9) R'= cI1*I1 + cI2*I2 + cH1*H1 + cH2*H2        
% (10) D'= d*(I2+H1) + dH2*H2
% (11) F'= fE+fT+b*I1
% 
% and 
%  a  is the infection rate
%  phi is the rate at which exposed become infectious
%  cX is the recovery rate, X=I1,I2,H1,H2
%  dX is the case fatality rate
%  b  is the rate at which infected become symptomatic
%  o  is the share of symptomatic that needs to he hospitalized
%  p2 is the share of hospitalized that needs ICU
%  s  relative infectiousness of symptomatic infected, s>=1
%  fTX tracing of newly symptomatic infected: previous contacts are
%     instantaneously identified and quarantined
%     fTX =eT*kappaX*b*I1
%     fTFX=eT*kappaX*fF   
%     with kappaX=avg new exposed and asymptomatic infectious from a newly
%     symptomatic individual 
%  fF unconditional testing with subsequent tracing
%     fF=(I1/(I1+S))*f
%  eX is the effectiveness of tracing and quarantine of identified individuals 
%      eX in [0,1], 1 being most effective
%
% Surviving cumulative infections are captured by the recovered population
% share. As long as the fatality rate is not too high. 
%
% Differential equation is defined for N-1 intervals [t_i,t_{i+1}] defined
% in vector ss=[t_1,t_2,...,t_N];
% The parameters (a,b,c) that defined the differential equation have to be
% of dimension (N-1)x1 
%
% A.Hornstein, 4/3/2020
% 4/11/2020 added separate tracking of exposed and asymptomatic
% 4/22/2020 added (1) option for high R0. (2) and comparison of actual and
% predicted cumulative deaths for first and second data point
% -------------------------------------------------------------------------
clear all;
KCclust=0;
figdir='GSEIRfig\';
tabdir='GSEIRtab\';      
matdir='GSEIRmat\';      
if KCclust==1
    figdir='GSEIRfig/';
    tabdir='GSEIRtab/';
    matdir='GSEIRmat/';      
end
% Update path to access procs_dir
utilities_dir = 'utilities';
pathO= path;
pathN= path(utilities_dir,pathO);

fid=1;

%% R0 option
% run it with standard R0=2.4 from IC2020 or high R0=5.7 from Sanche et al
% (2020): 'low' or 'high' 
R0opt='high'; 
% R0opt='low'; 

%% Policy paths proceed in nS phases. 
% The parameters below are defined as constant over all phases. The
% interventions change the values of some parameters
% The unit time interval is one year.
% Number of policy phases 
nS=3;
% Time paths proceed in 3 phases: [t1,t2],[t2,t3],[t3,tT]
w2m=round(52/12);
ssw=[0;2*w2m;5*w2m;10*52];
ss=ssw/52;

%% Parameterization
% baseline parameterization for comparative statics
GSEIRbasepar
% seeding the infection
GSEIRseedUSA
zName='USA';

%% construct time paths for the following variables
% solve differential equations on time interval t01 with initial conditions
% x0 for variable x
xlblS={'S','E','IA','ET','IAT','IS','HB','HICU','R','D','T'};
xlblL={'Susceptible','Exposed','Infected, Asymp','Exposed, Known','Infected, Asym Known', ...
    'Infected, Symp at Home','Infected, Symp in Hospital','Infected, Symp in ICU', ...
    'Recovered','Deaths','Test'};
I1=IC1-IC1Act;
I1e=0;
I2=IC1Act;
E =2*I1;
Ee=0;
x0=[ (pop-E-I1-I2);E;I1;Ee;I1e;(1-p.omg(1))*I2;  ...
    (1-p.eta(1))*p.omg(1)*I2; ...
    p.eta(1)*p.omg(1)*I2;0;F1c;0]/pop;

t01=[0;1.0];

%% define the following policy interventions
nC=6;
pp=cell(nC,1);
mlbl=cell(nC,1);

% -------------------------------------------------------------------------
% 0. No intervention 
iC=1;
pp{iC}=p;

mlbl{iC}='No Intervention';
mlblT{iC}=sprintf('Policy %1i',iC-1);

% -------------------------------------------------------------------------
% 1. Baseline intervention following IC is a temporary 3M quarantine and
% essentially permanent high-intensity SD. 
iC=iC+1;
pp{iC}=p;
% temporary inefficient quarantine
pp{iC}.eI2= [0.5; 1/3; 1/3];
% permanent SD: 45 percent reduction of basic reproduction rate
rdum=0.55*p.aS;
pp{iC}.aS=rdum;
pp{iC}.a =rdum.*p.aD;
[brr0,~]=GSEIR_R0fun(pp{iC});
pp{iC}.rho=(pp{iC}.a).*brr0;

mlbl{iC}='SD1Q1';
mlblT{iC}=sprintf('Policy %1i',iC-1);
% -------------------------------------------------------------------------
% 2. High-intensity SD for 1M that is relaxed over time, combined with
% permanent efficient quarantine
iC=iC+1;
pp{iC}=p;
% permanent efficient quarantine
pp{iC}.eI2= 0.9*ones(nS,1);
pp{iC}.eI1= 0.9*ones(nS,1);
pp{iC}.eT = 0.0*ones(nS,1);
% pp{iC}.eT = 1.00*ones(nS,1);
% relaxed SD: initial 45 percent reduction of basic reproduction rate
rdum=p.aS.*[0.55; 0.75; 0.95];
pp{iC}.aS=rdum;
pp{iC}.a =rdum.*p.aD;
[brr0,~]=GSEIR_R0fun(pp{iC});
pp{iC}.rho=pp{iC}.a.*brr0;

mlbl{iC}='SD2Q2';
mlblT{iC}=sprintf('Policy %1i',iC-1);
% -------------------------------------------------------------------------
% 3. High-intensity SD for 1M that is relaxed over time, combined with
% permanent efficient quarantine and tracing
iC=iC+1;
pp{iC}=p;
% permanent efficient quarantine and tracking
pp{iC}.eI2= 0.9*ones(nS,1);
pp{iC}.eI1= 0.9*ones(nS,1);
pp{iC}.eT = 0.9*ones(nS,1);
% relaxed SD: initial 45 percent reduction of basic reproduction rate
rdum=p.aS.*[0.55; 0.75; 0.95];
pp{iC}.aS=rdum;
pp{iC}.a =rdum.*p.aD;
[brr0,~]=GSEIR_R0fun(pp{iC});
pp{iC}.rho=pp{iC}.a.*brr0;

mlbl{iC}='SD2Q2T2';
mlblT{iC}=sprintf('Policy %1i',iC-1);

% -------------------------------------------------------------------------
% 4. High-intensity SD for 1M that is relaxed over time, combined with
% permanent efficient quarantine and testing
iC=iC+1;
pp{iC}=p;
% permanent efficient quarantine and tracking
pp{iC}.eI2= 0.9*ones(nS,1);
pp{iC}.eI1= 0.9*ones(nS,1);
pp{iC}.eT = 0.0*ones(nS,1);
% random testing of 100 percent of population in a year
pp{iC}.fE = 1.00*ones(nS,1);
% relaxed SD: initial 45 percent reduction of basic reproduction rate
rdum=p.aS.*[0.55; 0.75; 0.95];
pp{iC}.aS=rdum;
pp{iC}.a =rdum.*p.aD;
[brr0,~]=GSEIR_R0fun(pp{iC});
pp{iC}.rho=pp{iC}.a.*brr0;

mlbl{iC}='SD2Q2F2';
mlblT{iC}=sprintf('Policy %1i',iC-1);

% -------------------------------------------------------------------------
% 5. High-intensity SD for 1M that is relaxed over time, combined with
% permanent efficient quarantine, tracing, and testing
iC=iC+1;
pp{iC}=p;
% permanent efficient quarantine and tracking
pp{iC}.eI2= 0.9*ones(nS,1);
pp{iC}.eI1= 0.9*ones(nS,1);
pp{iC}.eT = 0.9*ones(nS,1);
% random testing of 100 percent of population in a year
pp{iC}.fE = 1.00*ones(nS,1);
% relaxed SD: initial 45 percent reduction of basic reproduction rate
rdum=p.aS.*[0.55; 0.75; 0.95];
pp{iC}.aS=rdum;
pp{iC}.a =rdum.*p.aD;
[brr0,~]=GSEIR_R0fun(pp{iC});
pp{iC}.rho=pp{iC}.a.*brr0;

mlbl{iC}='SD2Q2T2F2';
mlblT{iC}=sprintf('Policy %1i',iC-1);
% -------------------------------------------------------------------------
nC=iC;
mlbl=mlbl((1:nC));
mlblT=mlblT((1:nC));
cLbl='';
if ~isempty(R0opt)
    switch lower(R0opt)
        case 'high'
            cLbl='_R0High';
    end
end


%% construct time paths for the interventions
nx=length(x0);
xx=cell(nC,1);
tt=xx;
ww=xx;
dd=xx;
eH=xx;
ee=xx;
xxK=xx;
% -------------------------------------------------------------------------
for i=1:nC
    p=pp{i};
   [tt{i},xx{i}]=ode45(@(t,x)GSEIRdfeq(t,x,p,ss),t01,x0);
%   [tt{i},xx{i}]=ode15s(@(t,x)GSEIRdfeq(t,x,p,ss),t01,x0);
    xxK{i}=xx{i}*pop/10^3;
    ww{i}=tt{i}*52;
    dd{i}=tt{i}*365;
    [eH{i},ee{i}]=GSEIR_EMPfun(tt{i},xx{i},p,ss);
end
str_tmp = [matdir 'AltPol_', zName, cLbl ];
save(str_tmp,'tt','ss','pp','xx','xxK','xlblS','xlblL','mlbl','ee');

%% Plot variables for each case, population shares in percent
% save figures for ICU, Recovered, and Fatalities
%xslct=[ 7 8 9 ];
xslct=(1:11);
GSEIRfig_AltPol

%% Calculate summary statistics and print to TEX file
GSEIRstat_AltPol

%% Runup of deaths in first three weeks starting March 24th
GSEIRdeathsMarchApril