% GSEIR_Effectiveness_rho: Impact of social distancing
% This is a generalized SEIR model with hospitalization, fatalities, and
% quarantine/contact tracing/testing for time-varying parameters.
% Essentially this is a simplified version of the Imperial College
% pandemic model. Unless otherwise noted the model is calibrated based on
% the IC study. 
% 1 S:   susceptible population share
% 2 E:   exposed, not infectious, does not test positive
% 3 I1:  infected population share, asymptomatic
% 4 Ee:  known exposed
% 5 I1e: known asymptomatic infected
% 6 I2:  symptomatic infected, not hospitalized, but can be quarantined
% 7 H1:  symptomatic and hospitalized, not requiring ICU
% 8 H2:  symptomatic and hospitalized, requiring ICU
% 9 R:   recovered population share
% 10 D:  fatality share, cumulative
% 11 FE: total tests, cumulative 
%
% (0) M  = a*S*Ieff  and 
%     Ieff= I1 + (1-eTQ)*(Ee+I1e) + s*[(1-eI2)*I2 + (1-eH1)*H1 + (1-eH2)*H2] 
% (1) S' =-M
% (2) E' =+M - phi*E                        - fTE -fTFE
% (3) I1'=   + phi*E             -(b+cI1)*I1 - fTI1-fTFI1-fF 
% (4) Ee' =fTE +fTFE      -phi*Ee
% (5) I1e'=fTI1+fTFI1+ fF +phi*Ee-(b+cI1)*I1e
% (6) I2'=    (1-o)*b*(I1+I1e) -  (cI2+d)*I2 
% (7) H1'= (1-p2)*o*b*(I1+I1e) -  (cH1+d)*H1
% (8) H2'=     p2*o*b*(I1+I1e) -  (cH2+dH2)*H2
% (9) R'= cI1*I1 + cI2*I2 + cH1*H1 + cH2*H2        
% (10) D'= d*(I2+H1) + dH2*H2
% (11) F'= fE+fT+b*I1
% 
% and 
%  a  is the infection rate
%  phi is the rate at which exposed become infectious
%  cX is the recovery rate, X=I1,I2,H1,H2
%  dX is the case fatality rate
%  b  is the rate at which infected become symptomatic
%  o  is the share of symptomatic that needs to he hospitalized
%  p2 is the share of hospitalized that needs ICU
%  s  relative infectiousness of symptomatic infected, s>=1
%  fTX tracing of newly symptomatic infected: previous contacts are
%     instantaneously identified and quarantined
%     fTX =eT*kappaX*b*I1
%     fTFX=eT*kappaX*fF   
%     with kappaX=avg new exposed and asymptomatic infectious from a newly
%     symptomatic individual 
%  fF unconditional testing with subsequent tracing
%     fF=(I1/(I1+S))*f
%  eX is the effectiveness of tracing and quarantine of identified individuals 
%      eX in [0,1], 1 being most effective
%
% Surviving cumulative infections are captured by the recovered population
% share. As long as the fatality rate is not too high. 
%
% Differential equation is defined for N-1 intervals [t_i,t_{i+1}] defined
% in vector ss=[t_1,t_2,...,t_N];
% The parameters (a,b,c) that defined the differential equation have to be
% of dimension (N-1)x1 
%
% A.Hornstein, 3/20/2020
% 3/30/2020 fixed the calculation of basic reproduction rate and included
% employment impact of SD
% 3/31/2020 added an exposed state, testing and tracking, and total tests 
% 4/11/2020 added separate tracking of exposed and asymptomatic
% -------------------------------------------------------------------------
clear all;
KCclust=0;
figdir='GSEIRfig\';
tabdir='GSEIRtab\';      
matdir='GSEIRmat\';      
if KCclust==1
    figdir='GSEIRfig/';
    tabdir='GSEIRtab/';
    matdir='GSEIRmat/';      
end
% Update path to access procs_dir
utilities_dir = 'utilities';
pathO= path;
pathN= path(utilities_dir,pathO);

fid=1;
%% SD option: a=aD*aS
% wide SD=[1, 0.25] or narrow SD=[0.65 to 0.5]
% SDopt='narrow'; 
SDopt='wide';
%% R0 option
% run it with standard R0=2.4 from IC2020 or high R0=5.7 from Sanche et al
% (2020): 'low' or 'high' 
R0opt='low'; 

% -------------------------------------------------------------------------
% Time paths proceed in 2 phases
% 1. Temporary intervention from t1 to t2
% 2. Permanent modification from t2 to t3 
% The unit time interval is one year.
ssw=[0;6*round(52/12);10*52];
ss=ssw/52;
nS=length(ss)-1;

% baseline parameterization for comparative statics
GSEIRbasepar
% seeding the infection
GSEIRseedUK

%% construct time paths
% solve differential equations on time interval t01 with initial conditions
% x0 for variable x
xlblS={'S','E','IA','ET','IAT','IS','HB','HICU','R','D','T'};
xlblL={'Susceptible','Exposed','Infected, Asymp','Exposed, Known','Infected, Asym Known', ...
    'Infected, Symp at Home','Infected, Symp in Hospital','Infected, Symp in ICU', ...
    'Recovered','Deaths','Test'};
I1=IC1-IC1Act;
I1e=0;
I2=IC1Act;
E =1.5*I1;
Ee=0;
x0=[ (pop-E-I1-I2);E;I1;Ee;I1e;(1-p.omg(1))*I2;  ...
    (1-p.eta(1))*p.omg(1)*I2; ...
    p.eta(1)*p.omg(1)*I2;0;F1c;0]/pop;

t01=[0;1.5];
nx=length(x0);
% -------------------------------------------------------------------------
%% Sensitivity to permanent and transitory changes of basic reproduction rate
switch lower(SDopt)
    case 'wide'
        aS=[1; 0.75; 0.5; 0.25 ];
        aD=p.aD0*ones(4,1);
        aDS=aD.*aS;
        cLbl='';
    case 'narrow'
        aS=[0.65; 0.6; 0.55; 0.50 ];
        aD=p.aD0*ones(4,1);
        aDS=aD.*aS;
        cLbl='_AtOne';
end

[brr0,~]=GSEIR_R0fun(p);
zz=aDS.*brr0(1);
zLbl ='R_0';
zLblT='$\mathcal{R}_0$';
zName='R0';

nC=length(zz);
xx=cell(2*nC,1);
tt=xx;
ww=xx;
dd=xx;
ee=xx;
xxK=xx;
xxS=xx;
xxSK=xx;
pp=xx;
mlbl=cell(2*nC,1);
mlblT=mlbl;
for j=1:nC
    % permanent intervention
    i=j;
    p.a =aDS(j)*ones(nS,1);
    p.aS=aS(j)*ones(nS,1);
    p.aD=aD(j)*ones(nS,1);     
    [tt{i},xx{i}]=ode45(@(t,x)GSEIRdfeq(t,x,p,ss),t01,x0);
%     [tt{i},xx{i}]=ode15s(@(t,x)GSEIRdfeq(t,x,p,ss),t01,x0);
    xxK{i}=xx{i}*pop/10^3;
    pp{i}=p;
    ww{i}=tt{i}*52;
    dd{i}=tt{i}*365;
    ee{i}=GSEIR_EMPfun(tt{i},xx{i},p,ss);
    mlbl{i}=sprintf('%s=%4.2f, perm',zLbl,zz(j));
    mlblT{i}=sprintf('%s=%4.2f',zLblT,zz(j));
    % steady state for permanent intervention
    tT=0.99*t01(end);
    xS0=xx{i}(end,:);
    xxS{i}=fsolve(@(x)GSEIRdfeq(tT,x,p,ss),xS0);
    xxSK{i}=xxS{i}*pop/10^3;
    % temporary intervention
    i=nC+j;
    p.a=[aDS(j); aDS(1)*ones(nS-1,1)];   
    p.aS=[aS(j); aS(1)*ones(nS-1,1)];   
    p.aD=[aD(j); aD(1)*ones(nS-1,1)]; 
    [tt{i},xx{i}]=ode45(@(t,x)GSEIRdfeq(t,x,p,ss),t01,x0);
%     [tt{i},xx{i}]=ode15s(@(t,x)GSEIRdfeq(t,x,p,ss),t01,x0);
    xxK{i}=xx{i}*pop/10^3;
    pp{i}=p;
    ww{i}=tt{i}*52;
    dd{i}=tt{i}*365;
    ee{i}=GSEIR_EMPfun(tt{i},xx{i},p,ss);
    mlbl{i}=sprintf('%s=%4.2f, trans',zLbl,zz(j));
    mlblT{i}=sprintf('%s=%4.2f',zLblT,zz(j));
end
str_tmp = [matdir 'Eff_', zName, cLbl ];
save(str_tmp,'tt','ss','pp','xx','xxK','xxS','xxSK','xlblS','xlblL','mlbl','ee');

%% Plot variables for each case, population shares in percent
% save figures for ICU, Recovered, and Fatalities
% xslct=10;
% xslct=[ 7 8 9 ];
xslct=(1:nx)';
GSEIRfig

%% Calculate summary statistics and print to TEX file
GSEIRstat

