%% ------------------------------------------------------------------------
function dx=GSEIRdfeq(t,x,p,ss)
% GSEIRdfeq x=[S,E,I1,Ee,I1e,I2,H1,H2,R,D,F]
% 1 S:   susceptible population share
% 2 E:   exposed, not infectious, does not test positive
% 3 I1:  infected population share, asymptomatic
% 4 Ee:  known exposed
% 5 I1e: known asymptomatic infected
% 6 I2:  symptomatic infected, not hospitalized, but can be quarantined
% 7 H1:  symptomatic and hospitalized, not requiring ICU
% 8 H2:  symptomatic and hospitalized, requiring ICU
% 9 R:   recovered population share
% 10 D:  fatality share, cumulative
% 11 FE: total tests, cumulative 
%
% (0) M  = a*S*Ieff  and 
%     Ieff= I1 + (1-eTQ)*(Ee+I1e) + s*[(1-eI2)*I2 + (1-eH1)*H1 + (1-eH2)*H2] 
% (1) S' =-M
% (2) E' =+M - phi*E                         - fTE -fTFE
% (3) I1'=   + phi*E             -(cI1+b)*I1 - fTI1-fTFI1-fF 
% (4) Ee' =fTE +fTFE      -phi*Ee
% (5) I1e'=fTI1+fTFI1+ fF +phi*Ee-(cI1+b)*I1e
% (6) I2'=    (1-o)*b*(I1+I1e) -  (cI2+d)*I2 
% (7) H1'= (1-p2)*o*b*(I1+I1e) -  (cH1+d)*H1
% (8) H2'=     p2*o*b*(I1+I1e) -  (cH2+dH2)*H2
% (9) R'= cI1*(I1+I1e) + cI2*I2 + cH1*H1 + cH2*H2        
% (10) D'= d*(I2+H1) + dH2*H2
% (11) F'= fE+fT+b*I1
% 
% and 
%  a  is the infection rate
%  phi is the rate at which exposed become infectious
%  cX is the recovery rate, X=I1,I2,H1,H2
%  dX is the case fatality rate
%  b  is the rate at which infected become symptomatic
%  o  is the share of symptomatic that needs to he hospitalized
%  p2 is the share of hospitalized that needs ICU
%  s  relative infectiousness of symptomatic infected, s>=1
%  fTX tracing of newly symptomatic infected: previous contacts are
%     instantaneously identified and quarantined
%     fTX =eT*kappaX*b*I1
%     fTFX=eT*kappaX*fF   
%     with kappaX=avg new exposed and asymptomatic infectious from a newly
%     symptomatic individual 
%  fF unconditional testing with subsequent tracing
%     fF=(I1/(I1+S))*f
%  eX is the effectiveness of tracing and quarantine of identified individuals 
%      eX in [0,1], 1 being most effective
% A.Hornstein 4/3/2020
% 4/11/2020: added tracing of exposed
% 4/21/2020: cleaned up treatment of testing and tracing
% 5/6/2020: define a=aS*aD
% -------------------------------------------------------------------------
i=(ss(1:end-1)<=t)&(t<ss(2:end));
a  =p.aD(i)*p.aS(i);
phi=p.phi(i);
b  =p.b(i);
cI1=p.cI1(i);
cI2=p.cI2(i);
cH1=p.cH1(i);
cH2=p.cH2(i);
d  =p.d(i);
dH2=p.dH2(i);
omg=p.omg(i);
eta=p.eta(i);
sig=p.sig(i);
eI1=p.eI1(i);
eI2=p.eI2(i);
eH1=p.eH1(i);
eH2=p.eH2(i);
eT =p.eT(i);
fE =p.fE(i);

Ie=x(3) + (1-eI1)*x(5) + sig*( (1-eI2)*x(6) + (1-eH1)*x(7) + (1-eH2)*x(8) );
m =a*x(1)*Ie;
[~,brrTA,brrTE]=GSEIR_R0fun(p,i);
kapTA=a*brrTA*x(1);
kapTE=a*brrTE*x(1);
psiTA=kapTA;
psiTE=kapTE;
pF =x(3)/(x(1)+x(2)+x(3));

I12 =b*x(3);
I1e2=b*x(5);
fTE =eT*kapTE*I12;
fTA =eT*kapTA*I12;
fTFE=eT*psiTE*pF*fE;
fTFA=eT*psiTA*pF*fE;

dx=NaN(11,1);
dx(1)=-m; % S'
dx(2)= m-phi*x(2)                    -fTE-fTFE; % E'
dx(3)=   phi*x(2)-I12- cI1*x(3)-pF*fE-fTA-fTFA; % I1'
dx(4)=      fTE+fTFE - phi*x(4);                % Ee'
dx(5)=pF*fE+fTA+fTFA + phi*x(4)-I1e2 -cI1*x(5); %I1e'
dx(6)=     (1-omg)*(I12+I1e2)-    (cI2+d)*x(6); % I2'
dx(7)= (1-eta)*omg*(I12+I1e2)-    (cH1+d)*x(7); % H1'
dx(8)=     eta*omg*(I12+I1e2)-  (cH2+dH2)*x(8); % H2'
dx(9)= cI1*(x(3)+x(5)) + cI2*x(6)+cH1*x(7)+cH2*x(8); % R'
dx(10)=            d*(x(6)+x(7)) +dH2*x(8); % D'
dx(11)=I12+I1e2 + fE;

end
